@@ -6,13 +6,13 @@ Net::OSCAR::Constants -- internal Net::OSCAR constants
 
 package Net::OSCAR::Constants;
 
-$VERSION = '1.11';
-$REVISION = '$Revision: 1.1.6.9 $';
+$VERSION = '1.905';
+$REVISION = '$Revision: 1.4.2.7 $';
 
 use strict;
 use vars qw(@ISA @EXPORT $VERSION);
 use Scalar::Util qw(dualvar);
-use Net::OSCAR::Utility qw(tlv);
+use Net::OSCAR::TLV;
 require Exporter;
 @ISA = qw(Exporter);
 
@@ -20,8 +20,10 @@ require Exporter;
 	FLAP_CHAN_NEWCONN FLAP_CHAN_SNAC FLAP_CHAN_ERR FLAP_CHAN_CLOSE
 	CONNTYPE_LOGIN CONNTYPE_BOS CONNTYPE_ADMIN CONNTYPE_CHAT CONNTYPE_CHATNAV CONNTYPE_ICON CONNTYPE_DIRECT_IN CONNTYPE_DIRECT_OUT
 	MODBL_ACTION_ADD MODBL_ACTION_DEL MODBL_WHAT_BUDDY MODBL_WHAT_GROUP MODBL_WHAT_PERMIT MODBL_WHAT_DENY
-	OSCAR_CAPS OSCAR_CAPS_INVERSE OSCAR_TOOLDATA
-	GROUP_PERMIT GROUP_DENY BUDTYPES ENCODING ERRORS
+	OSCAR_CAPS OSCAR_CAPS_INVERSE OSCAR_CAPS_SHORT_INVERSE OSCAR_TOOLDATA
+	GROUP_PERMIT GROUP_DENY BUDTYPES ERRORS
+
+	ICQ_META_INFO ICQ_META_INFO_INVERSE
 );
 
 
@@ -95,36 +97,37 @@ use constant OSCAR_CAPS => {
 		"0xf2, 0xe7, 0xc7, 0xf4, 0xfe, 0xad, 0x4d, 0xfb, 0xb2, 0x35, 0x36, 0x79, 0x8b, 0xdf, 0x00, 0x00"))},
 	secureim => {description => "SecureIM encryption", value => pack("C*", map{hex($_)} split(/[ \t\n,]+/,
 		"0x09 0x46 0x01 0xff 0x4c 0x7f 0x11 0xd1 0x82 0x22 0x44 0x45 0x53 0x54 0x00 0x00"))},
+	video => {description => "A/V chat", value => pack("C*", map{hex($_)} split(/[ \t\n,]+/,
+		"0x09 0x46 0x01 0x05 0x4c 0x7f 0x11 0xd1 0x82 0x22 0x44 0x45 0x53 0x54 0x00 0x00"))},
 };
 use constant OSCAR_CAPS_INVERSE => { map { OSCAR_CAPS()->{$_}->{value} => $_ } keys %{OSCAR_CAPS()} };
+use constant OSCAR_CAPS_SHORT_INVERSE => { map { substr(OSCAR_CAPS()->{$_}->{value}, 2, 2) => $_ } keys %{OSCAR_CAPS()} };
 
 use constant OSCAR_TOOLDATA => tlv(
-	0x0001 => {version => 0x0003, toolid => 0x0110, toolversion => 0x0739},
-	0x0013 => {version => 0x0003, toolid => 0x0110, toolversion => 0x0739},
-	0x0002 => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
-	0x0003 => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
-	0x0004 => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
+	0x0001 => {version => 0x0004, toolid => 0x0110, toolversion => 0x08E5},
+	0x0013 => {version => 0x0003, toolid => 0x0110, toolversion => 0x08E5},
+	0x0002 => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
+	0x0003 => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
+	0x0004 => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
 	0x0005 => {version => 0x0001, toolid => 0x0001, toolversion => 0x0001, nobos => 1},
-	0x0006 => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
-	0x0007 => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
+	0x0006 => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
+	0x0007 => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
 	0x0008 => {version => 0x0001, toolid => 0x0104, toolversion => 0x0001},
-	0x0009 => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
-	0x000A => {version => 0x0001, toolid => 0x0110, toolversion => 0x0739},
-	0x000B => {version => 0x0001, toolid => 0x0104, toolversion => 0x0001},
+	0x0009 => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
+	0x000A => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
+	0x000B => {version => 0x0001, toolid => 0x0110, toolversion => 0x08E5},
 	0x000C => {version => 0x0001, toolid => 0x0104, toolversion => 0x0001, nobos => 1},
-	0x000D => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
-	0x000E => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
-	0x000F => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
-	0x0010 => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
+	0x000D => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
+	0x000E => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
+	0x000F => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
+	0x0010 => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
 	0x0015 => {version => 0x0001, toolid => 0x0110, toolversion => 0x047C, nobos => 1},
 	0x0017 => {version => 0x0000, toolid => 0x0000, toolversion => 0x0000, nobos => 1},
-	0x0018 => {version => 0x0001, toolid => 0x0010, toolversion => 0x0739, nobos => 1},
+	0x0018 => {version => 0x0001, toolid => 0x0010, toolversion => 0x08E5, nobos => 1},
 	0xFFFF => {version => 0x0000, toolid => 0x0000, toolversion => 0x0000, nobos => 1},
 );
 
-use constant BUDTYPES => ("buddy", "group", "permit entry", "deny entry", "visibility/misc. data", "presence", "unknown 6", "unknown 7", "unknown 8", "unknown 9", "unknown 10", "unknown 11", "unknown 12", "unknown 13", "unknown 14", "unknown 15", "unknown 16", "unknown 17", "unknown 18", "unknown 19", "buddy icon data");
-
-use constant ENCODING => 'text/aolrtf; charset="us-ascii"';
+use constant BUDTYPES => ("buddy", "group", "permit entry", "deny entry", "visibility/misc. data", "presence", undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, undef, "buddy icon data");
 
 use constant ERRORS => split(/\n/, <<EOF);
 Invalid error
@@ -159,4 +162,17 @@ Unknown error 28
 There have been too many recent signons from this address.  Please wait a few minutes and try again.
 EOF
 
+
+use constant ICQ_META_INFO => {
+	basic => 200,
+	office => 210,
+	background => 220,
+	notes => 230,
+	email => 235,
+	interests => 240,
+	affiliations => 250,
+	homepage => 270
+};
+use constant ICQ_META_INFO_INVERSE => { map { ICQ_META_INFO()->{$_} => $_ } keys %{ICQ_META_INFO()} };
+
 1;
