@@ -10,14 +10,15 @@ and the sending of buddylist changes to the OSCAR server.
 package Net::OSCAR::_BLInternal;
 
 use strict;
-use Net::OSCAR::OldPerl;
 use Net::OSCAR::Common qw(:all);
 use Net::OSCAR::Constants;
 use Net::OSCAR::Utility;
+use Net::OSCAR::TLV;
+use Net::OSCAR::XML;
 
 use vars qw($VERSION $REVISION);
-$VERSION = '1.11';
-$REVISION = '$Revision: 1.32.6.13 $';
+$VERSION = '1.905';
+$REVISION = '$Revision: 1.38.2.9 $';
 
 sub init_entry($$$$) {
 	my($blinternal, $type, $gid, $bid) = @_;
@@ -32,9 +33,6 @@ sub init_entry($$$$) {
 sub blparse($$) {
 	my($session, $data) = @_;
 
-	# This stuff was figured out more through sheer perversity
-	# than by actually understanding what all the random bits do.
-
 	$session->{visibility} = VISMODE_PERMITALL; # If we don't have p/d data, this is default.
 
 	delete $session->{blinternal};
@@ -51,7 +49,7 @@ sub blparse($$) {
 		while(my($key, $value) = each %$typedata) {
 			$session->{blinternal}->{$type}->{$gid}->{$bid}->{data}->{$key} = $value;
 		}
-		$session->log_printf(OSCAR_DBG_DEBUG, "Got BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $name, $type, $gid, $bid, $sublen, hexdump(tlv_encode($typedata)));
+		$session->log_printf_cond(OSCAR_DBG_DEBUG, sub { "Got BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $name, $type, $gid, $bid, $sublen, hexdump(tlv_encode($typedata)) });
 	}
 
 	return BLI_to_NO($session);
@@ -64,6 +62,8 @@ sub BLI_to_NO($) {
 	my($session) = @_;
 	my $bli = $session->{blinternal};
 
+	delete $session->{blinternal_visbid};
+	delete $session->{blinternal_iconbid};
 	delete $session->{buddies};
 	delete $session->{permit};
 	delete $session->{deny};
@@ -71,8 +71,6 @@ sub BLI_to_NO($) {
 	delete $session->{groupperms};
 	delete $session->{profile};
 	delete $session->{appdata};
-	delete $session->{showidle};
-	delete $session->{presence_unknown};
 
 	$session->{buddies} = bltie(1);
 	$session->{permit} = bltie;
@@ -91,7 +89,8 @@ sub BLI_to_NO($) {
 		}
 	}
 
-	if(exists $bli->{4} and (my($visbid) = keys %{$bli->{4}->{0}})) {
+	if(exists $bli->{4} and exists $bli->{4}->{0} and (my($visbid) = grep {exists($bli->{4}->{0}->{$_}->{data}->{0xCB})} keys %{$bli->{4}->{0}})) {
+		$session->{blinternal_visbid} = $visbid;
 		my $typedata = $bli->{4}->{0}->{$visbid}->{data};
 		($session->{visibility}) = unpack("C", $typedata->{0xCA}) if $typedata->{0xCA};
 
@@ -102,11 +101,6 @@ sub BLI_to_NO($) {
 		($session->{icon_timestamp}) = unpack("N", $typedata->{0x0102}) if exists($typedata->{0x0102});
 		($session->{icon_length}) = unpack("N", $typedata->{0x0103}) if exists($typedata->{0x0103});
 
-		delete $typedata->{0xCB};
-		delete $typedata->{0xCA};
-		delete $typedata->{0x0100};
-		delete $typedata->{0x0101};
-		delete $typedata->{0x0102};
 		$session->{appdata} = $typedata;
 
 		$session->set_info($session->{profile}) if exists($session->{profile});
@@ -116,20 +110,10 @@ sub BLI_to_NO($) {
 		$session->{groupperms} = 0xFFFFFFFF;
 	}
 
-	if(exists $bli->{5} and (my($presbid) = keys %{$bli->{5}->{0}})) {
-		my $typedata = $bli->{5}->{0}->{$presbid}->{data};
-		($session->{showidle}) = unpack("N", $typedata->{0xC9} || pack("F", 0x0061E7FF));
-		($session->{presence_unknown}) = unpack("N", $typedata->{0xD6} || pack("F", 0x0077FFFF));
-		delete $typedata->{0xC9};
-		delete $typedata->{0xD6};
-	} else {
-		# OSCAR complains if we set presence when it didn't give it to us...
-		#$session->{showidle} = 0x0061E7FF;
-		#$session->{presence_unknown} = 0x0077FFFF;
-	}
-
-	if(exists $bli->{0x14}) {
-		$session->{icon_md5sum} = $bli->{0x14}->{0}->{0x51F4}->{data}->{0xD5};
+	if(exists $bli->{0x14} and exists $bli->{0x14}->{0} and (my($iconbid) = grep {exists($bli->{0x14}->{0}->{$_}->{data}->{0xD5})} keys %{$bli->{0x14}->{0}})) {
+		$session->{blinternal_iconbid} = $iconbid;
+		my $typedata = $bli->{0x14}->{0}->{$iconbid}->{data};
+		$session->{icon_md5sum} = $typedata->{0xD5};
 	}
 
 	my @gids = unpack("n*", (exists($bli->{1}) and exists($bli->{1}->{0}) and exists($bli->{1}->{0}->{0}) and exists($bli->{1}->{0}->{0}->{data}->{0xC8})) ? $bli->{1}->{0}->{0}->{data}->{0xC8} : "");
@@ -158,7 +142,6 @@ sub BLI_to_NO($) {
 		$entry->{data} = $bli->{1}->{$gid}->{0}->{data};
 
 		my @bids = unpack("n*", $bli->{1}->{$gid}->{0}->{data}->{0xC8} || "");
-		delete $bli->{1}->{$gid}->{0}->{data}->{0xC8};
 
 		push @bids, grep { # Find everything...
 			my $inbud = $_;
@@ -175,11 +158,9 @@ sub BLI_to_NO($) {
 
 			my $comment = undef;
 			$comment = $buddy->{data}->{0x13C} if exists($buddy->{data}->{0x13C});
-			delete $buddy->{data}->{0x13C};
 
 			my $alias = undef;
 			$alias = $buddy->{data}->{0x131} if exists($buddy->{data}->{0x131});
-			delete $buddy->{data}->{0x131};
 
 			$session->{buddies}->{$group}->{members}->{$buddy->{name}} ||= {};
 			my $entry = $session->{buddies}->{$group}->{members}->{$buddy->{name}};
@@ -199,6 +180,27 @@ sub NO_to_BLI($) {
 	my $session = shift;
 
 	my $bli = tlv();
+	my $oldbli = $session->{blinternal};
+
+	# Copy old data
+	my $visbid = $session->{blinternal_visbid} || int(rand(30000)) + 1;
+	my $iconbid = $session->{blinternal_iconbid} || 0x51F4;
+	foreach my $type (keys %$oldbli) {
+		next if $type < 4;
+		foreach my $gid (keys %{$oldbli->{$type}}) {
+			foreach my $bid (keys %{$oldbli->{$type}->{$gid}}) {
+				next if $type == 4 and $bid == $visbid;
+				next if $type == 0x14 and $bid == $iconbid;
+
+				init_entry($bli, $type, $gid, $bid);
+				$bli->{$type}->{$gid}->{$bid}->{name} = $oldbli->{$type}->{$gid}->{$bid}->{name};
+				foreach my $data (keys %{$oldbli->{$type}->{$gid}->{$bid}->{data}}) {
+					$bli->{$type}->{$gid}->{$bid}->{data}->{$data} = $oldbli->{$type}->{$gid}->{$bid}->{data}->{$data};
+				}
+			}
+		}
+	}
+
 
 	foreach my $permit (keys %{$session->{permit}}) {
 		init_entry($bli, 2, 0, $session->{permit}->{$permit}->{buddyid});
@@ -210,37 +212,35 @@ sub NO_to_BLI($) {
 		$bli->{3}->{0}->{$session->{deny}->{$deny}->{buddyid}}->{name} = $deny;
 	}
 
-	my $vistype;
-	$vistype = (keys %{$session->{blinternal}->{4}->{0}})[0] if exists($session->{blinternal}->{4}) and exists($session->{blinternal}->{4}->{0}) and scalar keys %{$session->{blinternal}->{4}->{0}};
-	$vistype ||= int(rand(30000)) + 1;
-	init_entry($bli, 4, 0, $vistype);
-	$bli->{4}->{0}->{$vistype}->{data}->{0xCA} = pack("C", $session->{visibility} || VISMODE_PERMITALL);
-	$bli->{4}->{0}->{$vistype}->{data}->{0xCB} = pack("N", $session->{groupperms} || 0xFFFFFFFF);
+	init_entry($bli, 4, 0, $visbid);
+	$bli->{4}->{0}->{$visbid}->{data}->{0xCA} = pack("C", $session->{visibility} || VISMODE_PERMITALL);
+	$bli->{4}->{0}->{$visbid}->{data}->{0xCB} = pack("N", $session->{groupperms} || 0xFFFFFFFF);
 
 	#Net::OSCAR protocol extensions
-	$bli->{4}->{0}->{$vistype}->{data}->{0x0100} = $session->{profile} if $session->{profile};
-	$bli->{4}->{0}->{$vistype}->{data}->{0x0101} = pack("n", $session->{icon_checksum}) if $session->{icon_checksum};
-	$bli->{4}->{0}->{$vistype}->{data}->{0x0102} = pack("N", $session->{icon_timestamp}) if $session->{icon_timestamp};
-	$bli->{4}->{0}->{$vistype}->{data}->{0x0103} = pack("N", $session->{icon_length}) if $session->{icon_length};
+	$bli->{4}->{0}->{$visbid}->{data}->{0x0100} = $session->{profile} if $session->{profile};
+	$bli->{4}->{0}->{$visbid}->{data}->{0x0101} = pack("n", $session->{icon_checksum}) if $session->{icon_checksum};
+	$bli->{4}->{0}->{$visbid}->{data}->{0x0102} = pack("N", $session->{icon_timestamp}) if $session->{icon_timestamp};
+	$bli->{4}->{0}->{$visbid}->{data}->{0x0103} = pack("N", $session->{icon_length}) if $session->{icon_length};
 
 	foreach my $appdata(keys %{$session->{appdata}}) {
-		$bli->{4}->{0}->{$vistype}->{data}->{$appdata} = $session->{appdata}->{$appdata};
+		$bli->{4}->{0}->{$visbid}->{data}->{$appdata} = $session->{appdata}->{$appdata};
 	}
 
-	if(exists($session->{showidle})) {
-		my $presencetype;
-		$presencetype = (keys %{$session->{blinternal}->{5}->{0}})[0] if exists($session->{blinternal}->{5}) and exists($session->{blinternal}->{5}->{0}) and scalar keys %{$session->{blinternal}->{5}->{0}};
-		$presencetype ||= 0x0001;
-		init_entry($bli, 5, 0, $presencetype);
-		$bli->{5}->{0}->{$presencetype}->{data}->{0xC9} = pack("N", exists($session->{showidle}) ? $session->{showidle} : 0x0061E7FF);
-		$bli->{5}->{0}->{$presencetype}->{data}->{0xD6} = pack("N", exists($session->{presence_unknown}) ? $session->{presence_unknown} : 0x0077FFFF);
-	}
+	if(exists($session->{icon_md5sum}) || chain_exists($oldbli, 0x14, 0, $iconbid)) {
+		init_entry($bli, 0x14, 0, $iconbid);
 
+		if(chain_exists($oldbli, 0x14, 0, $iconbid)) {
+			$bli->{0x14}->{0}->{$iconbid}->{name} = $oldbli->{0x14}->{0}->{$iconbid}->{name};
 
-	if(exists($session->{icon_md5sum})) {
-		init_entry($bli, 0x14, 0, 0x51F4);
-		$bli->{0x14}->{0}->{0x51F4}->{name} = "1";
-		$bli->{0x14}->{0}->{0x51F4}->{data}->{0xD5} = $session->{icon_md5sum};
+			$bli->{0x14}->{0}->{$iconbid}->{data}->{$_} = $oldbli->{0x14}->{0}->{$iconbid}->{data}->{$_}
+			   foreach grep { $_ != 0xD5 } keys %{$oldbli->{0x14}->{0}->{$iconbid}->{data}};
+		} else {
+			$bli->{0x14}->{0}->{$iconbid}->{name} = "1";
+		}
+
+		if(exists($session->{icon_md5sum})) {
+			$bli->{0x14}->{0}->{$iconbid}->{data}->{0xD5} = $session->{icon_md5sum};
+		}
 	}
 
 	init_entry($bli, 1, 0, 0);
@@ -249,9 +249,18 @@ sub NO_to_BLI($) {
 		my $gid = $session->{buddies}->{$group}->{groupid};
 		init_entry($bli, 1, $gid, 0);
 		$bli->{1}->{$gid}->{0}->{name} = $group;
-		$bli->{1}->{$gid}->{0}->{data}->{0xC8} = pack("n*",
-			map { $_->{buddyid} }
-			values %{$session->{buddies}->{$group}->{members}});
+
+		# It seems that WinAIM can now have groups without 0xC8 data, and gets pissed if we create such data where it doesn't exist.
+		if(!exists($oldbli->{1}->{$gid}) or chain_exists($oldbli, 1, $gid, 0, "data", 0xC8)) {
+			$bli->{1}->{$gid}->{0}->{data}->{0xC8} = pack("n*",
+				map { $_->{buddyid} }
+				values %{$session->{buddies}->{$group}->{members}});
+		}
+
+		if(chain_exists($oldbli, 1, $gid, 0)) {
+			$bli->{1}->{$gid}->{0}->{data}->{$_} = $oldbli->{1}->{$gid}->{0}->{data}->{$_}
+			   foreach grep { $_ != 0xC8 } keys %{$oldbli->{1}->{$gid}->{0}->{data}};
+		}
 
 		foreach my $buddy(keys %{$session->{buddies}->{$group}->{members}}) {
 			my $bid = $session->{buddies}->{$group}->{members}->{$buddy}->{buddyid};
@@ -277,18 +286,27 @@ sub BLI_to_OSCAR($$) {
         $session->crapout($session->{services}->{0+CONNTYPE_BOS}, "You must wait for a buddylist_ok or buddylist_error callback before calling commit_buddylist again.") if $session->{budmods};
 	$session->{budmods} = [];
 
+	my %budmods;
+	$budmods{add} = [];
+	$budmods{modify} = [];
+	$budmods{delete} = [];
+
 	# First, delete stuff that we no longer use and modify everything else
 	foreach my $type(keys %$oldbli) {
+
+		my $budtype = (BUDTYPES)[$type] || "unknown type $type";
+
 		foreach my $gid(keys %{$oldbli->{$type}}) {
 			foreach my $bid(keys %{$oldbli->{$type}->{$gid}}) {
 				my $oldentry = $oldbli->{$type}->{$gid}->{$bid};
+
 				my $olddata = tlv_encode($oldentry->{data});
-				$session->log_printf(OSCAR_DBG_DEBUG, "Old BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $oldentry->{name}, $type, $gid, $bid, length($olddata), hexdump($olddata));
+				$session->log_printf_cond(OSCAR_DBG_DEBUG, sub { "Old BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $oldentry->{name}, $type, $gid, $bid, length($olddata), hexdump($olddata) });
 				my $delete = 0;
 				if(exists($newbli->{$type}) and exists($newbli->{$type}->{$gid}) and exists($newbli->{$type}->{$gid}->{$bid})) {
 					my $newentry = $newbli->{$type}->{$gid}->{$bid};
 					my $newdata = tlv_encode($newentry->{data});
-					$session->log_printf(OSCAR_DBG_DEBUG, "New BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $newentry->{name}, $type, $gid, $bid, length($newdata), hexdump($newdata));
+					$session->log_printf_cond(OSCAR_DBG_DEBUG, sub { "New BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $newentry->{name}, $type, $gid, $bid, length($newdata), hexdump($newdata) });
 
 					next if
 						$newentry->{name} eq $oldentry->{name}
@@ -300,18 +318,15 @@ sub BLI_to_OSCAR($$) {
 					} else {
 						$session->log_print(OSCAR_DBG_DEBUG, "Modifying.");
 
-						push @modifies, {
-							reqdata => {desc => "modifying ".(BUDTYPES)[$type]." $newentry->{name}", type => $type, gid => $gid, bid => $bid},
-							data =>
-								pack("na* nnn na*",
-									length($newentry->{name}),
-									$newentry->{name},
-									$gid,
-									$bid,
-									$type,
-									length($newdata),
-									$newdata
-								)
+						push @{$budmods{modify}}, {
+							reqdata => {desc => "modifying $budtype $newentry->{name}", type => $type, gid => $gid, bid => $bid},
+							protodata => {
+								entry_name => $newentry->{name},
+								group_id => $gid,
+								buddy_id => $bid,
+								entry_type => $type,
+								entry_data => $newdata
+							}
 						};
 					}
 				} else {
@@ -321,18 +336,15 @@ sub BLI_to_OSCAR($$) {
 				if($delete) {
 					$session->log_print(OSCAR_DBG_DEBUG, "Deleting.");
 
-					push @deletes, {
-						reqdata => {desc => "deleting ".(BUDTYPES)[$type]." $oldentry->{name}", type => $type, gid => $gid, bid => $bid},
-						data => 
-							pack("na* nnn na*",
-								length($oldentry->{name}),
-								$oldentry->{name},
-								$gid,
-								$bid,
-								$type,
-								length($olddata),
-								$olddata
-							)
+					push @{$budmods{delete}}, {
+						reqdata => {desc => "deleting $budtype $oldentry->{name}", type => $type, gid => $gid, bid => $bid},
+						protodata => {
+							entry_name => $oldentry->{name},
+							group_id => $gid,
+							buddy_id => $bid,
+							entry_type => $type,
+							entry_data => $olddata
+						}
 					};
 				}
 			}
@@ -341,26 +353,26 @@ sub BLI_to_OSCAR($$) {
 
 	# Now, add the new stuff
 	foreach my $type(keys %$newbli) {
+
+		my $budtype = (BUDTYPES)[$type] || "unknown type $type";
+
 		foreach my $gid(keys %{$newbli->{$type}}) {
 			foreach my $bid(keys %{$newbli->{$type}->{$gid}}) {
 				next if exists($oldbli->{$type}) and exists($oldbli->{$type}->{$gid}) and exists($oldbli->{$type}->{$gid}->{$bid}) and $oldbli->{$type}->{$gid}->{$bid}->{name} eq $newbli->{$type}->{$gid}->{$bid}->{name};
 				my $entry = $newbli->{$type}->{$gid}->{$bid};
 				my $data = tlv_encode($entry->{data});
 
-				$session->log_printf(OSCAR_DBG_DEBUG, "New BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $entry->{name}, $type, $gid, $bid, length($data), hexdump($data));
+				$session->log_printf_cond(OSCAR_DBG_DEBUG, sub { "New BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $entry->{name}, $type, $gid, $bid, length($data), hexdump($data) });
 
-				push @adds, {
-					reqdata => {desc => "adding ".(BUDTYPES)[$type]." $entry->{name}", type => $type, gid => $gid, bid => $bid},
-					data =>
-						pack("na* nnn na*",
-							length($entry->{name}),
-							$entry->{name},
-							$gid,
-							$bid,
-							$type,
-							length($data),
-							$data
-						)
+				push @{$budmods{add}}, {
+					reqdata => {desc => "adding $budtype $entry->{name}", type => $type, gid => $gid, bid => $bid},
+					protodata => {
+						entry_name => $entry->{name},
+						group_id => $gid,
+						buddy_id => $bid,
+						entry_type => $type,
+						entry_data => $data
+					}
 				};
 			}
 		}
@@ -368,24 +380,19 @@ sub BLI_to_OSCAR($$) {
 
 	# Actually send the changes.  Don't send more than 7K in a single SNAC.
 	# FLAP size limit is 8K, but that includes headers - good to have a safety margin
-	foreach my $type(0xA, 0x8, 0x9) {
-		my $changelist;
-		if($type == 0x8) {
-			$changelist = \@adds;
-		} elsif($type == 0x9) {
-			$changelist = \@modifies;
-		} else {
-			$changelist = \@deletes;
-		}
+	foreach my $type (qw(add modify delete)) {
+		my $changelist = $budmods{$type};
 
-		my($packet, @reqdata, @packets);
+		my(@reqdata, @packets);
+		my $packet = "";
 		foreach my $change(@$changelist) {
-			$packet .= $change->{data};
+			$packet .= protoparse($session, "buddylist modification")->pack(%{$change->{protodata}});
 			push @reqdata, $change->{reqdata};
 
 			if(length($packet) > 7*1024) {
 				#$session->log_print(OSCAR_DBG_INFO, "Adding to blmod queue (max packet size reached): type $type, payload size ", scalar(@reqdata));
 				push @packets, {
+					type => $type,
 					data => $packet,
 					reqdata => [@reqdata],
 				};
@@ -396,15 +403,22 @@ sub BLI_to_OSCAR($$) {
 		if($packet) {
 			#$session->log_print(OSCAR_DBG_INFO, "Adding to blmod queue (no more changes): type $type, payload size ", scalar(@reqdata));
 			push @packets, {
+				type => $type,
 				data => $packet,
 				reqdata => [@reqdata],
 			};
 		}
 
-		push @{$session->{budmods}}, map {{family => 0x13, subtype => $type, reqdata => $_->{reqdata}, data => $_->{data}}} @packets
+		push @{$session->{budmods}}, map {
+			{
+				protobit => "buddylist " . $_->{type},
+				reqdata => $_->{reqdata},
+				protodata => {mods => $_->{data}}
+			};
+		} @packets;
 	}
 
-	push @{$session->{budmods}}, {family => 0x13, subtype => 0x12}; # End BL mods
+	push @{$session->{budmods}}, {protobit => "end buddylist modifications"}; # End BL mods
 	#$session->log_print(OSCAR_DBG_INFO, "Adding terminator to blmod queue.");
 
 	$session->{blold} = $oldbli;
@@ -416,9 +430,24 @@ sub BLI_to_OSCAR($$) {
 		$session->callback_buddylist_ok();
 	} else {
 		#$session->log_print(OSCAR_DBG_INFO, "Non-empty blmod queue - sending initiator and first change packet.");
-		$session->svcdo(CONNTYPE_BOS, family => 0x13, subtype => 0x11); # Start buddylist modifications
+		$session->svcdo(CONNTYPE_BOS, protobit => "start buddylist modifications");
 		$session->svcdo(CONNTYPE_BOS, %{shift @{$session->{budmods}}}); # Send the first modification
 	}
 }
 
+sub chain_exists($@) {
+	my($tlv, @refs) = @_;
+
+	while(@refs) {
+		my $ref = shift @refs;
+		if(exists($tlv->{$ref})) {
+			$tlv = $tlv->{$ref};
+		} else {
+			return 0;
+		}
+	}
+
+	return $tlv ? 1 : 0;	
+}
+
 1;
